==> backports_asyncio_runner-1.0.0-py3-none-any.whl: backports/asyncio/runner/tasks.py <==
"""
We must override asyncio.tasks._PyTask to be able to patch some behavior in Python < 3.11 and allow
this backport to function. C-impl of the API is not in scope of this backport at this time.

In order to support this backport, we had to introduce some changes to asyncio.Task available in 3.11:

    1. contextvars.copy_context behavior change on __init__.
    2. _num_cancels_requested attribute
    3. cancelling() function
    4. uncancel() function

All of these are implemented in [CTask](https://github.com/python/cpython/blob/3.11/Modules/_asynciomodule.c)
as well, but the C implementations were not backported. Trying to provide 1-1 Task implementation in this
backport to Python 3.11 is not in scope and not needed.

For #1, since we can't mimic how `contextvars.copy_context` is used in Python 3.11 in the new __init__:

```
    if context is None:
        self._context = contextvars.copy_context()
    else:
        self._context = context
```

We mimic these in `Runner.run` by patching `contextvars.copy_context` temporarily to return
our copied context instead. This assures the contextvars are preserved when the runner runs.

"""
import asyncio.tasks
import sys
from asyncio import AbstractEventLoop
from typing import Coroutine, TypeVar, Any, Optional

_T = TypeVar("_T")


# See https://github.com/python/cpython/blob/3.11/Lib/asyncio/tasks.py


class Task(asyncio.tasks._PyTask):  # type: ignore[name-defined, misc]
    """A coroutine wrapped in a Future."""

    def __init__(
        self,
        coro: Coroutine[Any, Any, _T],
        *,
        loop: Optional[AbstractEventLoop] = None,
        name: Optional[str] = None,
    ) -> None:
        self._num_cancels_requested = 0
        # https://github.com/python/cpython/blob/3.11/Modules/_asynciomodule.c#L2026
        # self._context is patched in Runner.run() instead.
        super().__init__(coro, loop=loop, name=name)

    def cancel(self, msg: Optional[str] = None) -> bool:
        """Request that this task cancel itself.

        This arranges for a CancelledError to be thrown into the
        wrapped coroutine on the next cycle through the event loop.
        The coroutine then has a chance to clean up or even deny
        the request using try/except/finally.

        Unlike Future.cancel, this does not guarantee that the
        task will be cancelled: the exception might be caught and
        acted upon, delaying cancellation of the task or preventing
        cancellation completely.  The task may also return a value or
        raise a different exception.

        Immediately after this method is called, Task.cancelled() will
        not return True (unless the task was already cancelled).  A
        task will be marked as cancelled when the wrapped coroutine
        terminates with a CancelledError exception (even if cancel()
        was not called).

        This also increases the task's count of cancellation requests.
        """
        self._log_traceback = False
        if self.done():
            return False
        self._num_cancels_requested += 1
        # These two lines are controversial.  See discussion starting at
        # https://github.com/python/cpython/pull/31394#issuecomment-1053545331
        # Also remember that this is duplicated in _asynciomodule.c.
        # if self._num_cancels_requested > 1:
        #     return False
        if self._fut_waiter is not None:
            if sys.version_info >= (3, 9):
                if self._fut_waiter.cancel(msg=msg):
                    # Leave self._fut_waiter; it may be a Task that
                    # catches and ignores the cancellation so we may have
                    # to cancel it again later.
                    return True
            else:
                if self._fut_waiter.cancel():
                    return True
        # It must be the case that self.__step is already scheduled.
        self._must_cancel = True
        if sys.version_info >= (3, 9):
            self._cancel_message = msg
        return True

    def cancelling(self) -> int:
        """Return the count of the task's cancellation requests.

        This count is incremented when .cancel() is called
        and may be decremented using .uncancel().
        """
        return self._num_cancels_requested

    def uncancel(self) -> int:
        """Decrement the task's count of cancellation requests.

        This should be called by the party that called `cancel()` on the task
        beforehand.

        Returns the remaining number of cancellation requests.
        """
        if self._num_cancels_requested > 0:
            self._num_cancels_requested -= 1
        return self._num_cancels_requested

==> backports_asyncio_runner-1.0.2-py3-none-any.whl: backports/asyncio/runner/tasks.py <==
"""Minimal backported implementation of asyncio._PyTask from 3.11 compatible down to Python 3.8."""
import asyncio.tasks
import sys
from asyncio import AbstractEventLoop
from typing import Coroutine, TypeVar, Any, Optional

_T = TypeVar("_T")


# See https://github.com/python/cpython/blob/3.11/Lib/asyncio/tasks.py


class Task(asyncio.tasks._PyTask):  # type: ignore[name-defined, misc]
    """A coroutine wrapped in a Future."""

    def __init__(
        self,
        coro: Coroutine[Any, Any, _T],
        *,
        loop: Optional[AbstractEventLoop] = None,
        name: Optional[str] = None,
    ) -> None:
        self._num_cancels_requested = 0
        # https://github.com/python/cpython/blob/3.11/Modules/_asynciomodule.c#L2026
        # Backport Note: self._context is temporarily patched in Runner.run() instead.
        super().__init__(coro, loop=loop, name=name)

    def cancel(self, msg: Optional[str] = None) -> bool:
        """Request that this task cancel itself.

        This arranges for a CancelledError to be thrown into the
        wrapped coroutine on the next cycle through the event loop.
        The coroutine then has a chance to clean up or even deny
        the request using try/except/finally.

        Unlike Future.cancel, this does not guarantee that the
        task will be cancelled: the exception might be caught and
        acted upon, delaying cancellation of the task or preventing
        cancellation completely.  The task may also return a value or
        raise a different exception.

        Immediately after this method is called, Task.cancelled() will
        not return True (unless the task was already cancelled).  A
        task will be marked as cancelled when the wrapped coroutine
        terminates with a CancelledError exception (even if cancel()
        was not called).

        This also increases the task's count of cancellation requests.
        """
        self._log_traceback = False
        if self.done():
            return False
        self._num_cancels_requested += 1
        # These two lines are controversial.  See discussion starting at
        # https://github.com/python/cpython/pull/31394#issuecomment-1053545331
        # Also remember that this is duplicated in _asynciomodule.c.
        # if self._num_cancels_requested > 1:
        #     return False
        if self._fut_waiter is not None:
            if sys.version_info >= (3, 9):
                if self._fut_waiter.cancel(msg=msg):
                    # Leave self._fut_waiter; it may be a Task that
                    # catches and ignores the cancellation so we may have
                    # to cancel it again later.
                    return True
            else:
                if self._fut_waiter.cancel():
                    return True
        # It must be the case that self.__step is already scheduled.
        self._must_cancel = True
        if sys.version_info >= (3, 9):
            self._cancel_message = msg
        return True

    def cancelling(self) -> int:
        """Return the count of the task's cancellation requests.

        This count is incremented when .cancel() is called
        and may be decremented using .uncancel().
        """
        return self._num_cancels_requested

    def uncancel(self) -> int:
        """Decrement the task's count of cancellation requests.

        This should be called by the party that called `cancel()` on the task
        beforehand.

        Returns the remaining number of cancellation requests.
        """
        if self._num_cancels_requested > 0:
            self._num_cancels_requested -= 1
        return self._num_cancels_requested
